/* DVB-T USB2 firmware for Cypress EZ-USB FX2
 * FX2 usb utilities
 */

#ifndef __DELAY_H__
#define __DELAY_H__

#include <fx2.h>

/**
 *  busy waiting
 */
extern void mdelay(short milliseconds);


/* access to the following registers must be separated by a synchronization
 * delay (c.f. section 15.14 of the FX2 user's manual):
 *
 * FIFORESET FIFOPINPOLAR INPKTEND EPxBCH:L EPxFIFOPFH:L EPxAUTOINLENH:L
 * EPxFIFOCFG EPxGPIFFLGSEL PINFLAGSAB PINFLAGSCD EPxFIFOIE EPxFIFOIRQ GPIFIE
 * GPIFIRQ UDMACRCH:L GPIFADRH:L GPIFTRIG EPxGPIFTRIG OUTPKTEND REVCTL GPIFTCB3
 * GPIFTCB2 GPIFTCB1 GPIFTCB0
 *
 * The following definition is for CLKOUT = IFCLK = 48MHz *only*.
 */

#define syncdelay() 			\
	do {				\
		_asm nop _endasm;	\
		_asm nop _endasm;	\
		_asm nop _endasm;	\
	} while (0)


#endif /* __DELAY_H__ */

