/* DVB-T USB2 firmware for Cypress EZ-USB FX2
 * descriptor tables
 */

#ifndef __DESCRIPTORS_H__
#define __DESCRIPTORS_H__

#include "types.h"

/* Standard requests, for the bRequest field of a SETUP packet */
#define USB_REQ_GET_STATUS              0x00
#define USB_REQ_CLEAR_FEATURE           0x01
#define USB_REQ_SET_FEATURE             0x03
#define USB_REQ_SET_ADDRESS             0x05
#define USB_REQ_GET_DESCRIPTOR          0x06
#define USB_REQ_SET_DESCRIPTOR          0x07
#define USB_REQ_GET_CONFIGURATION       0x08
#define USB_REQ_SET_CONFIGURATION       0x09
#define USB_REQ_GET_INTERFACE           0x0A
#define USB_REQ_SET_INTERFACE           0x0B
#define USB_REQ_SYNCH_FRAME             0x0C

/* Descriptor types ... USB 2.0 spec table 9.5 */
#define USB_DT_DEVICE                   0x01
#define USB_DT_CONFIG                   0x02
#define USB_DT_STRING                   0x03
#define USB_DT_INTERFACE                0x04
#define USB_DT_ENDPOINT                 0x05
#define USB_DT_DEVICE_QUALIFIER         0x06
#define USB_DT_OTHER_SPEED_CONFIG       0x07
#define USB_DT_INTERFACE_POWER          0x08

/* feature request types */
#define USB_FT_ENDPOINT_HALT		0
#define USB_FT_DEVICE_REMOTE_WAKEUP	1
#define USB_FT_TEST_MODE		2


extern xdata u8 usb_desc_device[];
extern xdata u8 usb_desc_device_qualifier[];
extern xdata u8 usb_desc_config_full_speed[];
extern xdata u8 usb_desc_config_high_speed[];

extern xdata u8 usb_desc_string_language[];
extern xdata u8 usb_desc_string_manufacturer[];
extern xdata u8 usb_desc_string_product[];

#endif /* __DESCRIPTORS_H__ */
