; DVB-T USB2 firmware for Cypress EZ-USB FX2
; descriptor tables

; Note: asx8051 stores 16-bit words as big-endian, but descriptors
; must have them as little endian.
; to be save we don't use .dw but .db only

	.module descriptors

VENDOR_ID	= 0x4711	; FIXME
PRODUCT_ID	= 0x0023	; FIXME
PRODUCT_VERSION	= 0x0100	; 1.0

; some definitons were taken from
; /usr/src/linux-2.5.54/include/linux/usb_ch9.h


; Descriptor types ... USB 2.0 spec table 9.5
USB_DT_DEVICE			= 0x01
USB_DT_CONFIG			= 0x02
USB_DT_STRING			= 0x03
USB_DT_INTERFACE		= 0x04
USB_DT_ENDPOINT			= 0x05
USB_DT_DEVICE_QUALIFIER		= 0x06
USB_DT_OTHER_SPEED_CONFIG	= 0x07
USB_DT_INTERFACE_POWER		= 0x08

USB_DT_DEVICE_SIZE		= 18
USB_DT_CONFIG_SIZE		= 9
USB_DT_INTERFACE_SIZE		= 9
USB_DT_ENDPOINT_SIZE		= 7
USB_DT_DEVICE_QUALIFIER_SIZE	= 10

; endpoint types
USB_ET_CONTROL			= 0
USB_ET_ISO			= 1
USB_ET_BULK			= 2
USB_ET_INT			= 3

; isochronous endpoint attributes
USB_ET_UNSYNC			= (0 << 2)
USB_ET_ASYNC			= (1 << 2)
USB_ET_ADAPTIVE			= (2 << 2)
USB_ET_SYNC			= (3 << 2)
USB_ET_DATA			= (0 << 4)
USB_ET_FEEDBACK			= (1 << 4)
USB_ET_DATA_FB			= (2 << 4)

; USB directions
USB_DIR_OUT			= 0
USB_DIR_IN			= 0x80


;; -----------------------------------------------------------------------------

;; put descriptor table right behind code and PID table
;; note: we dynamically change some descriptors, so they must
;; be declared as xdata in descriptors.h (code is const)
	.area	XSEG	(CODE,ABS)
	.org    0x400

_usb_desc_device::
	.db	USB_DT_DEVICE_SIZE
	.db	USB_DT_DEVICE
	.db	0, 2		; USB version 2.0 (BCD)
	.db	0xff		; device class
	.db	0xff		; sub class
	.db	0xff		; protocol (0xff == vendor-specific)
	.db	64		; EP0 max packet size
	.db	VENDOR_ID & 0xff, VENDOR_ID >> 8
	.db	PRODUCT_ID & 0xff, PRODUCT_ID >> 8
	.db	PRODUCT_VERSION & 0xff, PRODUCT_VERSION >> 8
	.db	1		; manufacturer string idx
	.db	2		; product string idx
	.db	0		; serial number string idx
	.db	1		; number of configurations

_usb_desc_device_qualifier::
	.db	USB_DT_DEVICE_QUALIFIER_SIZE
	.db	USB_DT_DEVICE_QUALIFIER
	.db	0, 2		; USB version 2.0 (BCD)
	.db	0xff		; device class
	.db	0xff		; sub class
	.db	0xff		; protocol (0xff == vendor-specific)
	.db	64		; EP0 max packet size
	.db	1		; number of configurations
	.db	0		; reserved

_usb_desc_config_high_speed::
	.db	USB_DT_CONFIG_SIZE
	.db	USB_DT_CONFIG
	.db	(usb_desc_config_high_speed_end - _usb_desc_config_high_speed) & 0xff
	.db	(usb_desc_config_high_speed_end - _usb_desc_config_high_speed) >> 8
	.db	1		; number of interfaces
	.db	1		; configuration number
	.db	0		; configuration string idx
	.db	0xe0		; flags: self-powered, remote wakeup
	.db	250		; FIXME: bus power in 2mA units

;; interface 0 / alternate setting 0
	.db	USB_DT_INTERFACE_SIZE
	.db	USB_DT_INTERFACE
	.db	0		; interface number
	.db	0		; alternate setting
	.db	4		; number of endpoints
	.db	0xff		; interface class (0xff == vendor specific)
	.db	0xff		; sub-class
	.db	0		; protocol
	.db	3		; interface string idx

;; JTAG
	.db	USB_DT_ENDPOINT_SIZE
	.db	USB_DT_ENDPOINT
	.db	(1 | USB_DIR_OUT)	; EP number and direction
	.db	USB_ET_BULK
	.db	64			; maximum packet size (lsb)
	.db	0			; maximum packet size (msb)
	.db	0			; polling interval

;; JTAG
	.db	USB_DT_ENDPOINT_SIZE
	.db	USB_DT_ENDPOINT
	.db	(1 | USB_DIR_IN)	; EP number and direction
	.db	USB_ET_BULK
	.db	64			; maximum packet size (lsb)
	.db	0			; maximum packet size (msb)
	.db	0			; polling interval

;; Data TO HOST
	.db	USB_DT_ENDPOINT_SIZE
	.db	USB_DT_ENDPOINT
	.db	(2 | USB_DIR_IN)	; EP number and direction
	.db	USB_ET_BULK
	.db	0			; maximum packet size (lsb)
	.db	2			; maximum packet size (msb)
	.db	0			; polling interval

;; Data FROM HOST
	.db	USB_DT_ENDPOINT_SIZE
	.db	USB_DT_ENDPOINT
	.db	(6 | USB_DIR_OUT)	; EP number and direction
	.db	USB_ET_BULK
	.db	0			; maximum packet size (lsb)
	.db	2			; maximum packet size (msb)
	.db	0			; polling interval

usb_desc_config_high_speed_end:   


; alignment
        .even

_usb_desc_config_full_speed::
	.db	USB_DT_CONFIG_SIZE
	.db	USB_DT_CONFIG
	.db	(usb_desc_config_full_speed_end - _usb_desc_config_full_speed) & 0xff
	.db	(usb_desc_config_full_speed_end - _usb_desc_config_full_speed) >> 8
	.db	1		; number of interfaces
	.db	1		; configuration number
	.db	0		; configuration string idx
	.db	0xe0		; flags: self-powered, remote wakeup
	.db	250		; FIXME: bus power in 2mA units

;; interface 0 / alternate setting 0
	.db	USB_DT_INTERFACE_SIZE
	.db	USB_DT_INTERFACE
	.db	0		; interface number
	.db	0		; alternate setting
	.db	4		; number of endpoints
	.db	0xff		; interface class (0xff == vendor specific)
	.db	0xff		; sub-class
	.db	0xff		; protocol
	.db	3		; interface string idx

;; JTAG
	.db	USB_DT_ENDPOINT_SIZE
	.db	USB_DT_ENDPOINT
	.db	(1 | USB_DIR_OUT)	; EP number and direction
	.db	USB_ET_BULK
	.db	64			; maximum packet size (lsb)
	.db	0			; maximum packet size (msb)
	.db	0			; polling interval

;; JTAG
	.db	USB_DT_ENDPOINT_SIZE
	.db	USB_DT_ENDPOINT
	.db	(1 | USB_DIR_IN)	; EP number and direction
	.db	USB_ET_BULK
	.db	64			; maximum packet size (lsb)
	.db	0			; maximum packet size (msb)
	.db	0			; polling interval

;; Data TO HOST
	.db	USB_DT_ENDPOINT_SIZE
	.db	USB_DT_ENDPOINT
	.db	(2 | USB_DIR_IN)	; EP number and direction
	.db	USB_ET_BULK
	.db	64			; maximum packet size (lsb)
	.db	0			; maximum packet size (msb)
	.db	0			; polling interval

;; Data FROM HOST
	.db	USB_DT_ENDPOINT_SIZE
	.db	USB_DT_ENDPOINT
	.db	(6 | USB_DIR_OUT)	; EP number and direction
	.db	USB_ET_BULK
	.db	64			; maximum packet size (lsb)
	.db	0			; maximum packet size (msb)
	.db	0			; polling interval

usb_desc_config_full_speed_end:   

; alignment
        .even

; note: strings are UNICODE
_usb_desc_string_language::
	.db	4		; length
	.db	USB_DT_STRING
	.db	0x09, 0x04	; FIXME: UNICODE language ID

; string 1
_usb_desc_string_manufacturer::
	.db	(usb_desc_string_manufacturer_end - _usb_desc_string_manufacturer)
	.db	USB_DT_STRING
	.db	'Z, 0
	.db	'N, 0
	.db	'U, 0
	.db	'H, 0
usb_desc_string_manufacturer_end:

; string 2
_usb_desc_string_product::
	.db	(usb_desc_string_product_end - _usb_desc_string_product)
	.db	USB_DT_STRING
	.db	'O, 0
	.db	'p, 0
	.db	'e, 0
	.db	'n, 0
	.db	'N, 0
	.db	'e, 0
	.db	'x, 0
	.db	'y, 0
	.db	's, 0
usb_desc_string_product_end:

; eof
