/* DVB-T USB2 firmware for Cypress EZ-USB FX2
 * Cypress EZ-USB FX2 register definitions
 */

#ifndef __FX2_H__
#define __FX2_H__

/********************************************************************************
 * "standard" microcontroller stuff
 ********************************************************************************/

//#define MCS51REG_ENABLE_WARNINGS
#define MICROCONTROLLER_CYPRESS_FX2

/* derived from MICROCONTROLLER_DS80C32X definition, modified for fx2 */
#define MICROCONTROLLER_DEFINED
#ifdef MCS51REG_ENABLE_WARNINGS
#warning Selected HW: Cypress EZ-USB FX2
#endif
// 8051 register set
#define P0
#define SP
#define DPL
#define DPH
#define PCON__SMOD__SMOD0__x__x__GF1__GF0__STOP__IDLE
#define TCON
#define TMOD
#define TL0
#define TL1
#define TH0
#define TH1
#define P1
#define SCON
#define SCON0
#define SBUF
#define P2
#define IE__EA__ES1__ET2__ES__ET1__EX1__ET0__EX0
#define P3
#define IP__x__PS1__PT2__PS__PT1_PX1__PT0__PX0
#define PSW
#define ACC
#define B
// 8052 specific registers
#define T2CON__TF2__EXF2__RCLK__TCLK__EXEN2__TR2__C_T2__CP_RL2
#define RCAP2L
#define RCAP2H
#define TL2
#define TH2
// DS80C320 specific register
#define DPL1
#define DPH1
#define DPS__x__x__x__x__x__x__x__SEL
#define SCON1
#define SBUF1

/* pull in standard definitions from sdcc library */
#include <mcs51reg.h>

/* additional definitions for fx2: */

sfr at 0x8E CKCON       ; // DS80C320 & DS80C390 specific
// Not directly accessible Bits.
#define MD0    0x01
#define MD1    0x02
#define MD2    0x04
#define T0M    0x08
#define T1M    0x10
#define T2M    0x20
#define WD0    0x40
#define WD1    0x80

/* external irq enable */
sfr at 0xE8 EIE  ;
sbit at 0xE8 EUSB   ;
sbit at 0xE9 EI2C   ;
sbit at 0xEA EX4    ;
sbit at 0xEB EX5    ;
sbit at 0xEC EX6    ;

/* external irq priority */
sfr at 0xF8 EIP  ;
sbit at 0xF8 PUSB   ;
sbit at 0xF9 PI2C   ;
sbit at 0xFA PX4    ;
sbit at 0xFB PX5    ;
sbit at 0xFC PX6    ;

/* external irq flags */
sfr at 0x91 EXIF  ;
#define USBINT 0x10
#define I2CINT 0x20
#define IE4    0x40
#define IE5    0x80

/********************************************************************************
 * USB / GPIF stuff
 ********************************************************************************/

sfr at 0x80 IOA     ;
sfr at 0x90 IOB     ;
sfr at 0x92 MPAGE   ;

sfr at 0x9A AUTOPTRH1  ;
sfr at 0x9B AUTOPTRL1  ;
sfr at 0x9D AUTOPTRH2  ;
sfr at 0x9E AUTOPTRL2  ;
sfr at 0xA0 IOC        ;
sfr at 0xA1 INT2CLR    ;
sfr at 0xA2 INT4CLR    ;

sfr at 0xAA EP2468STAT ;
#define EP2EMPTY 0x01
#define EP2FULL  0x02
#define EP4EMPTY 0x04
#define EP4FULL  0x08
#define EP6EMPTY 0x10
#define EP6FULL  0x20
#define EP8EMPTY 0x40
#define EP8FULL  0x80

sfr at 0xAB EP24FIFOFLGS   ;
sfr at 0xAC EP68FIFOFLGS   ;

sfr at 0xAF AUTOPTRSETUP  ;
sbit at 0xAF+0 EXTACC  ;
sbit at 0xAF+1 APTR1FZ ;
sbit at 0xAF+2 APTR2FZ ;

sfr at 0xB0 IOD     ;
sfr at 0xB1 IOE     ;
sfr at 0xB2 OEA     ;
sfr at 0xB3 OEB     ;
sfr at 0xB4 OEC     ;
sfr at 0xB5 OED     ;
sfr at 0xB6 OEE     ;

sfr at 0xBA EP01STAT    ;
sfr at 0xBB GPIFTRIG    ;

sfr at 0xBD GPIFSGLDATH     ;
sfr at 0xBE GPIFSGLDATLX    ;
sfr at 0xBF GPIFSGLDATLNOX  ;

sfr at 0xD8 EICON  ;
sbit at 0xD8+3 INT6  ;
sbit at 0xD8+4 RESI  ;
sbit at 0xD8+5 ERESI ;
sbit at 0xD8+7 SMOD1 ;

#include "fx2regs.h"

#endif /* __FX2_H__ */
