/***************************************************************************
 *            jtag.c
 *
 *  Sat Nov 24 18:33:00 2007
 *  Copyright  2007  User
 *  Email
 ****************************************************************************/

#include <fx2.h>
#include "delay.h"
#include "types.h"
#include "jtag.h"

void jtag_init() {

	/* pin 5 of port d disables tdi -> tdo forward */
	OED|=(1<<5);
	IOD|=(1<<5);

	/* set JTAG in/outputs */
	OED |= (TDI | TCK | TMS);
	OED &= ~TDO;

	/* set them low */
	IOD &= ~(TCK | TMS | TDI);
}

void set_tms(u8 val) {
	if(val)
		IOD|= TMS;
	else
		IOD&= ~TMS;
}

#define set_tdi(a)		IOD&= ~TDI; IOD|=(a)<<2;
#define set_tck(a)		IOD&= ~TCK; IOD|=(a)<<4;
#define tck_cycle()		set_tck(1); set_tck(0)

u8 handle_jtag(void) {	
	u16 bitlen = ( (EP1OUTBUF[0] & 1) << 8) | EP1OUTBUF[1];
	u8 bytecnt, byte, bitcnt;
		
	EP1INBUF[0] = EP1OUTBUF[0];
	EP1INBUF[1] = EP1OUTBUF[1];
	
	// TMS
	if(EP1OUTBUF[0] & CMD_JTAG_TMS) {
				
		for(bytecnt=0; bitlen; bytecnt++) {
			byte = EP1OUTBUF[2+bytecnt];
			
			// copy
			EP1INBUF[2+bytecnt] = byte;
			
			for(bitcnt=0; (bitlen) && (bitcnt<8); bitcnt++, bitlen--) {
				set_tms(byte&1);
				byte>>=1;
				
				tck_cycle();
			} // foreach bit
			
		} // foreach byte
		
	}
	
	// TDI/TDO
	else {
		u8 obyte;
		
		for(bytecnt=0; bitlen; bytecnt++) {
			byte = EP1OUTBUF[2+bytecnt];
			obyte=0;
			
			for(bitcnt=0; (bitlen) && (bitcnt<8); bitcnt++) {
				
				bitlen--;
				
				/* last one */
				if((!bitlen) && (EP1OUTBUF[0] & CMD_JTAG_LAST))
					set_tms(1);
				
				//set_tdi(byte&1);
				IOD&= ~TDI;
				IOD |= (byte&1)<<2;
				
				byte>>=1;
				
				obyte |= (IOD & TDO) << bitcnt;
				
				tck_cycle();
			} // foreach bit
		
			EP1INBUF[2+bytecnt] = obyte;
			
		} // foreach byte
			
	} // TDI/TDO
	
	return 64;
}
