
#define ALLOCATE_REGISTERS
#include <fx2.h>

#include "irq.h"
#include "bug.h"
#include "delay.h"
#include "jtag.h"
#include "usb_ep0.h"
#include "usb_ep1.h"

void power_init() {

	/* pin 7 of port d connected to mosfet gate controlling the board power
	 * ref: http://digilentinc.com/Data/Products/NEXYS/Nexys_sch.pdf
	 */
	
	/* configure pin 7 of port d as output */
	OED |= (1<<7);
	//SYNCDELAY;

}

void toggle_power() {

	/* toggle high/low state of the mosfet gate */

	if((IOD&(1<<7)))
		IOD&=~(1<<7);
	else
		IOD|=(1<<7);

	//SYNCDELAY;

}

#define HIGHSPEED	(USBCS & 0x80)

void slave_fifo_init() {

	/* initialization of the slave fifo, used by external logic (the fpga)
	 * to do usb communication with the host */
	EP2FIFOCFG=0;
	syncdelay();
	EP4FIFOCFG=0;
	syncdelay();
	EP6FIFOCFG=0;
	syncdelay();
	EP8FIFOCFG=0;
	syncdelay();

	EP2CFG=0;
	syncdelay();
	EP4CFG=0;
	syncdelay();
	EP6CFG=0;
	syncdelay();
	EP8CFG=0;
	syncdelay();
	
	/* set bit 0 and 1 - fifo slave config */	
	IFCONFIG=0xcb;
 	syncdelay();

	/* p. 180: must be set to 1 */
	REVCTL=0;//((1<<0)|(1<<1));
	syncdelay();

	EP6CFG= 0xa0;
	syncdelay();

	EP2CFG= 0xe0;
	syncdelay();	

	/* reset the fifo */
	FIFORESET=0x80; /* nak all transfers */
	syncdelay();

	FIFORESET=0x02; /* reset ep2 */
	syncdelay();

	FIFORESET=0x04;
	syncdelay();	

	FIFORESET=0x06; /* reset ep6 */
	syncdelay();

	FIFORESET=0x08;
	syncdelay();
	
	FIFORESET=0x00; /* restore normal operation */
	syncdelay();

	/* 8 bit fifo to all endpoints
	 *
	 * ('or' of all these bits define port d functionality)
	 */
	EP2FIFOCFG=(1<<3);
	syncdelay();
	
	/* auto in/out, no cpu interaction! auto in len = 64 - TODO: 512 with Hi speed */
	EP6FIFOCFG=0x10;
	syncdelay();

	OUTPKTEND=0x86;
	syncdelay();
	OUTPKTEND=0x86;
	syncdelay();
	OUTPKTEND=0x86;
	syncdelay();
	OUTPKTEND=0x86;
	syncdelay();

	if(HIGHSPEED) {
		EP2AUTOINLENH=2;
		syncdelay();
		EP2AUTOINLENL=0;
		syncdelay();
	}
	else {
		EP2AUTOINLENH=0;
		syncdelay();
		EP2AUTOINLENL=64;
		syncdelay();
	}
}

void main (void)
{
	u16 i;

	/* set data memory stretch to 0 (internal RAM is fast...)
	 *
	 * potentially dangerous, i2c doesn't seem to work reliably when
	 * using fast accesses...
	 *
	 *CKCON &= ~0x07;
	 */

	/* disable autovectoring for now */
	INTSETUP = 0x00;

	/* Enable 8051 interrupts */
	EA = 0;

	/* Enable both autopointers
	 * AUTOPTR1 is used for EP0 memory dump ioctl 0xa3 and 
	 * can be used within the main loop,
	 * AUTOPTR2 is intended for use in EP2FIFO interrupts only.
	 */
        //AUTOPTRSETUP = 0x7;
        
        /* run with 48Mhz */
        CPUCS = (CPUCS & 0x18) | 0x10;

	usb_ep0_init();
	usb_ep1_init();

	/* power init & power on */
	power_init();
	toggle_power();

	/* jtag init */
	jtag_init();

	slave_fifo_init();
	
	for (i=0; ;i++) {
		usb_check_suspend_state();
		usb_ep0_process_request();
		usb_ep1_process_request();

	}
}
