#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <usb.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <assert.h>

static usb_dev_handle *handle;

#define USB_ERROR_NOTFOUND  1
#define USB_ERROR_ACCESS    2
#define USB_ERROR_IO        3

#define USB_VENDOR_ID 0x4711
#define USB_PRODUCT_ID 0x0023

struct usb_device* find_znuhtag_device() {
	struct usb_bus* bus;
	struct usb_device* dev;
    int i;

	usb_find_busses();
	usb_find_devices();

    for (bus = usb_get_busses(); bus; bus = bus->next) 
	{
		if (bus->root_dev)
		{
			for (i = 0; i < bus->root_dev->num_children; i++)
		        if (bus->root_dev->children[i]->descriptor.idVendor == USB_VENDOR_ID &&
                    bus->root_dev->children[i]->descriptor.idProduct == USB_PRODUCT_ID)
					// Found it
					return bus->root_dev->children[i];
		}
		else
		{
			for (dev = bus->devices; dev; dev = dev->next)
                if (dev->descriptor.idVendor == USB_VENDOR_ID &&
                    dev->descriptor.idProduct == USB_PRODUCT_ID)
					// Found it
					return dev;
		}
    }

	return NULL;
}

int fickn(void) {
	struct usb_device* dev;

	dev = find_znuhtag_device();
	if (dev == NULL)
		goto cleanup;
        
	handle = usb_open(dev);
    if (handle == NULL)
        goto cleanup;

    if (usb_set_configuration(handle, 1) < 0)
        goto cleanup;

    if (usb_claim_interface(handle, 0) < 0)
        goto cleanup;
	
	return 0;
	
cleanup:
	assert(0);
	
	return 0;
}

unsigned long bytes=0;

void handler(int sig) {
	printf("%ld bytes/sec\n",bytes/10);
	bytes=0;
	alarm(10);
}

int main(int argc, char **argv) {
	unsigned char *buf;
	int ret, size=32*1024;

	if(argc>1) {
		if(atoi(argv[1])>0)
			size=atoi(argv[1])*1024;
	}

	buf=malloc(size);
	
	usb_init();
	
	fickn();

	printf("blocksize: %d\n",size);

	signal(SIGALRM,&handler);
	if(argc>1)
		alarm(10);
	
	do {
		ret = usb_bulk_read(handle, 0x82, (char *) buf, size, 5000);
		bytes+=ret;
		if(argc<2) {
			printf("\r%d: %02x",ret,buf[0]);
			fflush(stdout);
		}
	} while(ret>0);
	
	return 0;
}
