#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <usb.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <assert.h>

static usb_dev_handle *handle;

#define USB_ERROR_NOTFOUND  1
#define USB_ERROR_ACCESS    2
#define USB_ERROR_IO        3

#define USB_VENDOR_ID 0x4711
#define USB_PRODUCT_ID 0x0023

struct usb_device* find_znuhtag_device() {
	struct usb_bus* bus;
	struct usb_device* dev;
    int i;

	usb_find_busses();
	usb_find_devices();

    for (bus = usb_get_busses(); bus; bus = bus->next) 
	{
		if (bus->root_dev)
		{
			for (i = 0; i < bus->root_dev->num_children; i++)
		        if (bus->root_dev->children[i]->descriptor.idVendor == USB_VENDOR_ID &&
                    bus->root_dev->children[i]->descriptor.idProduct == USB_PRODUCT_ID)
					// Found it
					return bus->root_dev->children[i];
		}
		else
		{
			for (dev = bus->devices; dev; dev = dev->next)
                if (dev->descriptor.idVendor == USB_VENDOR_ID &&
                    dev->descriptor.idProduct == USB_PRODUCT_ID)
					// Found it
					return dev;
		}
    }

	return NULL;
}

int fickn(void) {
	struct usb_device* dev;

	dev = find_znuhtag_device();
	if (dev == NULL)
		goto cleanup;
        
	handle = usb_open(dev);
    if (handle == NULL)
        goto cleanup;

    if (usb_set_configuration(handle, 1) < 0)
        goto cleanup;

    if (usb_claim_interface(handle, 0) < 0)
        goto cleanup;
	
	return 0;
	
cleanup:
	assert(0);
	
	return 0;
}

int main(int argc, char **argv) {
	unsigned char buf[64];
	int ret;
	
	usb_init();
	
	fickn();

	memset(buf,0x23,64);

	do {
		ret = usb_bulk_write(handle, 6, (char *) buf, 64, 5000);
		buf[63]++;
		printf("%d\n",ret);
		usleep(10000);
	} while(ret==64);
	
	return 0;
}
